@@ -1,6 +1,7 @@
 """
 Unit tests for reverse URL lookups.
 """
+
 import pickle
 import sys
 import threading
@@ -1455,7 +1456,7 @@ class RequestURLconfTests(SimpleTestCase):
 
 
 class ErrorHandlerResolutionTests(SimpleTestCase):
-    """Tests for handler400, handler404 and handler500"""
+    """Tests for handler400, handler403, handler404 and handler500"""
 
     def setUp(self):
         urlconf = "urlpatterns_reverse.urls_error_handlers"
@@ -1464,12 +1465,12 @@ class ErrorHandlerResolutionTests(SimpleTestCase):
         self.callable_resolver = URLResolver(RegexPattern(r"^$"), urlconf_callables)
 
     def test_named_handlers(self):
-        for code in [400, 404, 500]:
+        for code in [400, 403, 404, 500]:
             with self.subTest(code=code):
                 self.assertEqual(self.resolver.resolve_error_handler(code), empty_view)
 
     def test_callable_handlers(self):
-        for code in [400, 404, 500]:
+        for code in [400, 403, 404, 500]:
             with self.subTest(code=code):
                 self.assertEqual(
                     self.callable_resolver.resolve_error_handler(code), empty_view
